@@ -13,10 +13,7 @@ use XML::Simple;
 use Time::HiRes;
 use Carp;
 
-our $VERSION = '0.03';
-
-our $DEFAULT_CHUNK = 250;
-our $DEFAULT_DV = 0;
+our $VERSION = '0.10';
 
 =head1 NAME
 
@@ -129,7 +126,7 @@ Update a record.
     
 =head1 DESCRIPTION
 
-This module provides an alternate Perl API for ServiceNow.
+This module provides access to ServiceNow via SOAP Web Services.
 
 Features of this module include:
 
@@ -147,14 +144,14 @@ documentation.
 
 =item *
 
-Robust, easy to use methods for reading large amounts of data
+Robust methods for reading large amounts of data
 which adhere to ServiceNow's L<best practice recommendations|http://wiki.servicenow.com/index.php?title=Web_Services_Integrations_Best_Practices#Queries>.
 
 =back
 
 =cut
 
-our $xmlns = 'http://www.glidesoft.com/';
+our $XMLNS = 'http://www.glidesoft.com/';
 our $context; # global variable points to current session
 
 sub SOAP::Transport::HTTP::Client::get_basic_credentials {
@@ -174,7 +171,7 @@ sub _params {
         my $value = shift;
         warn "Parameter name appears to be a sys_id"
             if $name =~ /^[0-9A-Fa-f]{32}$/;
-        push @params, SOAP::Data->name($name, $value);
+        push @params, SOAP::Data->name($name, $value)->type('string');
     }
     if (@_) {
         # there is one parameter left
@@ -202,22 +199,24 @@ sub ServiceNow {
 This C<ServiceNow> function
 (which is essentially an alias for C<ServiceNow::SOAP::Session-E<gt>new()>)
 is used to obtain a reference to a Session object. 
-The Session object essentially holds the URL and 
-connection credentials for your ServiceNow instance. 
+The Session object essentially holds the URL and
+connection credentials for your ServiceNow instance,
+and the L<SOAP::Lite> object.
 
-The first argument is the URL or instance name.
+The first argument to this function is the instance,
+which can be either a fully qualified URL
+(I<e.g.> C<"https://mycompanydev.service-now.com">)
+or an instance name (I<e.g.> C<"mycompanydev">).
 The second argument is the user name.
-The third argument is the password. 
+The third argument is the password.
 
-The fourth (optional) argument is an integer trace level
-which can be helpful for debugging. For details refer to 
-L</DIAGNOSTICS> below.
+Various options can be specified as
+name value pairs following the password.
+For a list of options refer to L</set>.
 
 B<Syntax>
 
-    my $sn = ServiceNow($instancename, $username, $password);
-    my $sn = ServiceNow($instanceurl, $username, $password);
-    my $sn = ServiceNow($instancename, $username, $password, $tracelevel);
+    my $sn = ServiceNow($instance, $username, $password, %options);
     
 B<Examples>
 
@@ -225,12 +224,13 @@ The following two statements are equivalent.
 
     my $sn = ServiceNow("https://mycompanydev.service-now.com", "soap.perl", $password);
     my $sn = ServiceNow("mycompanydev", "soap.perl", $password);
-    
-Tracing of Web Services calls can be enabled by passing in a fourth parameter.
-(See L</DIAGNOSTICS> below)
 
-    my $sn = ServiceNow("mycompanydev", "soap.perl", $password, 1);
+Various options can be specified as name/value pairs following the password.
+Refer to L</set> for details.
 
+    my $sn = ServiceNow("mycompanydev", $username, $password, 
+        query => 100000, trace => 1);
+        
 =head1 ServiceNow::SOAP::Session
 
 =cut
@@ -238,32 +238,39 @@ Tracing of Web Services calls can be enabled by passing in a fourth parameter.
 package ServiceNow::SOAP::Session;
 
 sub new {
-    my ($pkg, $url, $user, $pass, $trace) = @_;
+    my ($pkg, $url, $user, $pass, @options) = @_;
+    Carp::croak "Missing instance" unless $url;
+    Carp::carp "Username is blank" unless $user;
+    Carp::carp "Password is blank" unless $pass;
     # strip off any trailing slash
     $url =~ s/\/$//; 
     # append '.service-now.com' unless the URL contains a '.'
     $url = $url . '.service-now.com' unless $url =~ /\./;
     # prefix with 'https://' unless the URL already starts with https:
     $url = 'https://' . $url unless $url =~ /^http[s]?:/;
+    # This endpoint is a stub.  Endpoint will be changed before each call.
+    my $endpoint = "$url/sys_user.do?SOAP";
+    my $cookiejar = HTTP::Cookies->new(ignore_discard => 1);
+    my @params = (cookie_jar => $cookiejar);
+    my $client = SOAP::Lite->proxy($endpoint, @params);
     my $session = {
         url => $url,
-        cookie_jar => HTTP::Cookies->new(ignore_discard => 1),
         user => $user, 
         pass => $pass,
-        trace => $trace || 0,
+        trace => 0,
+        dv => 0,
+        fetch => 250,
+        query => 0,
+        client => $client
     };
     bless $session, $pkg;
-    $context = $session;
+    $session->set(@options);
+    my $trace = $session->{trace};
     print "url=$url; user=$user\n" if $trace;
+    $context = $session;
     return $session;
 }
 
-sub setTrace {
-    my ($self, $trace) = @_;
-    $self->{trace} = $trace;
-    return $self;
-}
-
 our $startTime;
 
 sub traceBefore {
@@ -276,15 +283,15 @@ sub traceBefore {
 }
 
 sub traceAfter {
-    my ($self, $trace, $client, $message, $content) = @_;
+    my ($self, $trace, $message, $content) = @_;
     return unless $trace;
     my $finishTime = Time::HiRes::time();
     my $elapsed = $finishTime - $startTime;
     print sprintf(" %.2fs ", $elapsed), $message, "\n";
     if ($trace > 1) {
-        print defined $content ? $content : 
-            $client->transport()->http_response()->content(),
-            "\n";
+        $content = $self->{client}->transport()->http_response()->content()
+            unless defined $content;
+        print $content, "\n";
     }
 }
 
@@ -320,17 +327,16 @@ sub call {
     my $trace = $self->{trace};
     my $baseurl = $self->{url};
     my $endpoint = "$baseurl/$function.do?SOAP";
-    my $cookie_jar = $self->{cookie_jar};
-    my $client = SOAP::Lite->proxy($endpoint, cookie_jar => $cookie_jar);
+    my $client = $self->{client};
     my @params = 
         (@_ && ref $_[0] eq 'HASH') ?
         ServiceNow::SOAP::_params(%{$_[0]}) :
         ServiceNow::SOAP::_params(@_);
     $self->traceBefore($trace, $function);
     $context = $self;
-    my $som = $client->call('execute' => @params);
+    my $som = $client->endpoint($endpoint)->call('execute' => @params);
     Carp::croak $som->faultdetail if $som->fault;
-    $self->traceAfter($trace, $client);
+    $self->traceAfter($trace);
     my $response = $som->body->{executeResponse};
     if (ref $response eq 'HASH') {
         return wantarray ? %{$response} : $response;
@@ -371,7 +377,7 @@ sub connect {
 
 =head2 loadSession
 
-This method loads the session information (i.e. cookies) from a file.
+This method loads the session information (I<i.e.> cookies) from a file.
 This may be required if you are running the same perl script multiple times
 (each time in a separate process)
 and you do not want to establish a separate ServiceNow session
@@ -393,7 +399,7 @@ sub loadSession {
 
 =head2 saveSession
 
-This method saves the session information (i.e. cookies) to a file.  
+This method saves the session information (I<i.e.> cookies) to a file.  
 For usage notes refer to L</loadSession> above.
 
 B<Syntax>
@@ -408,6 +414,97 @@ sub saveSession {
     $cookie_jar->save($filename);
 }
 
+=head2 set
+
+This method permits assignment of various default values and processing options.
+This method is generally not required
+since the same options can be specified 
+following the password parameter of L</ServiceNow>.
+
+The following options may be specified.
+
+=over
+
+=item *
+
+C<dv> - Specify a default value for Display Values.
+This can be overridden at the Table level by L</setDV>.
+For an explanation of values see L</setDV>.
+This option is only effective if it is changed
+prior to calling the L</table> method.
+
+=item *
+
+C<fetch> - Specify default number of records to be retrieved
+by L</fetch>. Default is 250.
+This option will be ignored if it is overridden
+by a parameter to L</fetch> or L</fetchAll>.
+
+=item *
+
+C<query> - Specify the number of keys to be retrieved by
+L</query> in each call to L</getKeys>.
+The default is 0,
+which causes all key to be retrieved 
+in a single call.
+
+=item *
+
+C<timeout> - Specify a 
+SOAP::Transport::HTTP::Client timeout value.
+
+=item *
+
+C<trace> - Specify a trace level.
+Refer to L</DIAGNOSTICS> below.
+Default is 0.
+
+=back
+
+B<Syntax>
+
+    $sn->set(%options);
+    
+B<Example>
+
+    $sn->set(query => 100000, trace => 1);
+
+=cut
+
+sub set {
+    my $self = shift;
+    while (@_) {
+        my $key = shift;
+        my $value = shift;
+        Carp::croak '"fetch" must be greater than 0'
+            if $key eq "fetch" and $value < 1;
+        Carp::croak "option \"$key\" not recognized"
+            unless $key =~ /^(dv|fetch|query|timeout|trace)$/;                       
+        $self->{$key} = $value;
+        if ($key eq 'timeout' and $value > 0) {
+            $self->{client}->transport->timeout($value);
+        }        
+    }
+    return $self;
+}    
+
+sub setTrace {
+    my ($self, $trace) = @_;
+    $self->{trace} = $trace;
+    return $self;
+}
+
+=head2 soap
+
+This method returns a reference to the underlying SOAP::Lite object.
+
+=cut
+
+sub soap {
+    my $self = shift;
+    return $self->{client};
+}
+
 =head2 table
 
 Used to obtain a reference to a Table object.  
@@ -416,7 +513,7 @@ L</ServiceNow::SOAP::Table> methods described below.
 
 B<Syntax>
 
-    my $table = $sn->table($tablename);
+    $table = $sn->table($tablename);
 
 B<Example>
 
@@ -435,13 +532,11 @@ sub table {
 
 Table objects are used for ServiceNow's
 L<Direct Web Services API|http://wiki.servicenow.com/index.php?title=SOAP_Direct_Web_Service_API>.
-A Table object holds the URI for a SOAP endpoint
-and the corresponding SOAP::Lite object.
 
 To obtain a Table object,
 use the L</table> method describe above.
 
-These Table object methods provide an interface to the Direct Web Services API:
+These Table object methods provide access to the Direct Web Services API:
 L</deleteRecord>,
 L</get>,
 L</getKeys>,
@@ -475,16 +570,12 @@ sub new {
     my ($pkg, $session, $name) = @_;
     $context = $session;
     my $baseurl = $session->{url}; 
-    my $cookie_jar = $session->{cookie_jar};
     my $endpoint = "$baseurl/$name.do?SOAP";
-    my $client = SOAP::Lite->proxy($endpoint, cookie_jar => $cookie_jar);
     my $new = bless {
         session => $session,
         name => $name,
         endpoint => $endpoint,
-        client => $client,
-        dv => $DEFAULT_DV,
-        chunk => $DEFAULT_CHUNK,
+        dv => $session->{dv},
         trace => $session->{trace}
     }, $pkg;
     return $new;
@@ -493,17 +584,17 @@ sub new {
 sub callMethod {
     my $self = shift;
     my $methodname = shift;
-    my $trace = $self->{trace};
-    my $baseurl = $self->{session}->{url};
     my $tablename = $self->{name};
+    my $session = $self->{session};
+    my $client = $session->{client};
+    my $baseurl = $session->{url};
     my $endpoint = "$baseurl/$tablename.do?SOAP";
     if (($methodname eq 'get' || $methodname eq 'getRecords') && $self->{dv}) {
         my $dv = $self->{dv};
         $endpoint = "$baseurl/$tablename.do?displayvalue=" . $dv . "&SOAP";
     }
     $context = $self->{session};
-    my $method = SOAP::Data->name($methodname)->attr({xmlns => $xmlns});
-    my $client = $self->{client};
+    my $method = SOAP::Data->name($methodname)->attr({xmlns => $XMLNS});
     # my @params = ServiceNow::SOAP::_params(@_);
     my $som = $client->endpoint($endpoint)->call($method => @_);
     Carp::croak $som->faultdetail if $som->fault;        
@@ -512,20 +603,19 @@ sub callMethod {
 
 sub traceBefore {
     my ($self, $methodname) = @_;
-    my $trace = $self->{trace};
-    return unless $trace;
     my $session = $self->{session};
+    my $trace = $session->{trace};
+    return unless $trace;
     my $tablename = $self->{name};
     $session->traceBefore($trace, "$tablename $methodname");
 }
 
 sub traceAfter {
     my ($self, $message, $content) = @_;
-    my $trace = $self->{trace};
-    return unless $trace;
     my $session = $self->{session};
-    my $client = $self->{client};
-    $session->traceAfter($trace, $client, $message, $content);
+    my $trace = $session->{trace};
+    return unless $trace;
+    $session->traceAfter($trace, $message, $content);
 }
 
 =head2 asQuery
@@ -534,11 +624,11 @@ This method creates a new L<Query|/ServiceNow::SOAP::Query> object
 from a list of keys.
 It does not make any Web Services calls.
 It simply makes a copy of the list.
-Each item in the list must be a B<sys_id> for the respective table.
+Each item in the list must be a sys_id for the respective table.
 
 B<Syntax>
 
-    my $query = $table->asQuery(@keys);
+    $query = $table->asQuery(@keys);
 
 B<Example>
 
@@ -662,9 +752,9 @@ L<Aggregate Web Service plugin|http://wiki.servicenow.com/index.php?title=SOAP_D
 
 B<Syntax >
 
-    my $count = $table->count();
-    my $count = $table->count(%parameters);
-    my $count = $table->count($encodedquery);
+    $count = $table->count();
+    $count = $table->count(%parameters);
+    $count = $table->count($encodedquery);
 
 B<Examples>
 
@@ -690,7 +780,7 @@ sub count {
 
 =head2 deleteRecord
 
-Deletes a record.
+This method deletes a record using the sys_id.
 For information on available parameters
 refer to the ServiceNow documentation on the 
 L<"deleteRecord" Direct SOAP API method|http://wiki.servicenow.com/index.php?title=SOAP_Direct_Web_Service_API#deleteRecord>.
@@ -719,7 +809,7 @@ This method returns a list of all columns in the table
 B<except> those in the argument list.
 The argument(s) to this method can be either a string 
 containing a comma separated list of names
-or a list of names (i.e. C<qw>).
+or a list of names (I<i.e.> C<qw>).
 This method returns a string containing a comma separated list of names.
 
 The [only useful] purpose of this function
@@ -729,15 +819,15 @@ L<extended query parameter|http://wiki.servicenow.com/index.php?title=Direct_Web
 
 B<Syntax>
 
-    my $names = $table->except(@list_of_columns);
-    my $names = $table->except($comma_separated_list_of_columns);
+    $names = $table->except(@list_of_columns);
+    $names = $table->except($comma_separated_list_of_columns);
     
 B<Example>
 
 This example retrieves the columns sys_id, number and description
 from the incident table.
 In other words, 
-the result excludes all columns except sys_id, number and description.
+the result excludes all columns B<except> sys_id, number and description.
 
     my $incident = $sn->table("incident");
     my @recs = $incident->getRecords(
@@ -924,7 +1014,7 @@ sub getRecords {
 =head2 insert
 
 This method inserts a record.
-In a scalar context, this method returns a B<sys_id> only.
+In a scalar context, this method returns a sys_id only.
 In a list context, this method returns a list of name/value pairs.
 
 For information on available parameters
@@ -933,14 +1023,14 @@ L<"insert" Direct SOAP API method|http://wiki.servicenow.com/index.php?title=SOA
 
 B<Syntax>
 
-    my $sys_id = $table->insert(%values);
-    my %result = $table->insert(%values);
+    $sys_id = $table->insert(%values);
+    %result = $table->insert(%values);
     
 B<Examples>
 
     my $incident = $sn->table("incident");
 
-In a scalar context, the function returns a B<sys_id>.
+In a scalar context, the function returns a sys_id.
 
     my $sys_id = $incident->insert(
         short_description => $short_description,
@@ -995,7 +1085,7 @@ L<"insertMultiple" Direct SOAP API method|http://wiki.servicenow.com/index.php?t
 
 B<Syntax>
 
-    my @results = $table->insertMultiple(@records);
+    @results = $table->insertMultiple(@records);
 
 B<Example>
 
@@ -1078,7 +1168,7 @@ sub update {
     if (_isGUID($_[0])) { unshift @_, 'sys_id' };
     my %values = @_;
     my $sysid = $values{sys_id};
-    $self->traceBefore("update $sysid");
+    $self->traceBefore("update");
     my $som = $self->callMethod('update', _params @_);
     $self->traceAfter();
     return $self;
@@ -1088,11 +1178,22 @@ sub update {
 
 This method creates a new L<Query|/ServiceNow::SOAP::Query> object
 by calling L</getKeys>.
+You can then use L</fetch> or L</fetchAll>
+to retrieve the actual record.
+
+This method normally fetches all keys using a single 
+call to L</getKeys>.
+However, if the number of keys is too large you may
+encounter timeout errors. In this situation the recommendation
+is to fetch the keys in chunks by specifying 
+a C<query> option in the L</ServiceNow> function.
+The suggested initial setting is C<query =E<gt> 100000>.
+If timeouts are still a problem the reduce the value further.
 
 B<Syntax>
 
-    my $query = $table->query(%parameters);
-    my $query = $table->query($encodedquery);
+    $query = $table->query(%parameters);
+    $query = $table->query($encodedquery);
     
 B<Example>
 
@@ -1107,15 +1208,42 @@ until all records have been retrieved.
         __order_by => "sys_created_on");
     my @recs = $qry->fetchAll();
 
+The following example retrieves all keys for the C<cmdb_ci>
+table in chunks of 100,000 at a time.
+
+    my $sn = ServiceNow($instance, $user, $pass, query => 100000);
+    my $cmdb_ci = $sn->table("cmdb_ci");
+    my $qry = $cmdb_ci->query();
+
 =cut
 
 sub query {
-    my $self = shift;
-    my $query = ServiceNow::SOAP::Query->new($self);
-    my @keys = $self->getKeys(@_);
+    my $table = shift;
+    my $session = $table->{session};
+    my $limit = $session->{query};
+    my $query = ServiceNow::SOAP::Query->new($table);
+    # if there is a single parameter it must be an encoded query
+    if (@_ == 1) { unshift @_, '__encoded_query' };
+    my %params = @_;
+    my @keys;
+    if ($limit) {
+        my $first = 0;
+        my $last = $limit;
+        while ($first <= @keys) {
+            my %p = %params;
+            $p{__first_row} = $first;
+            $p{__last_row} = $last;
+            my @k = $table->getKeys(%p);
+            push @keys, @k;
+            $first += $limit;
+            $last = $first + $limit;
+        }
+    }
+    else {
+        @keys = $table->getKeys(%params);
+    }
     # parameters which affect order or columns must be preserved
     # so that they can be passed to getRecords
-    my %params = @_;
     for my $k (keys %params) {
         delete $params{$k} unless $k =~ /^(__order|__exclude|__use_view)/;
     }
@@ -1223,17 +1351,20 @@ L<"Return Display Value for Reference Variables"|http://wiki.servicenow.com/inde
 
 B<Syntax>
 
-    $table->setDV("true");
-    $table->setDV("all");
-    $table->setDv("");  # restore default setting
+    $table->setDv("");     # return sys_id only (default)
+    $table->setDV("true"); # return display value instead of sys_id
+    $table->setDV("all");  # return both sys_id and display value
+    $table->setDV(0);      # same as ""
+    $table->setDV(1);      # same as "true"
+    $table->setDV(2);      # same as "all"
 
 B<Examples>
 
-    my $sys_user_group = $session->table("sys_user_group")->setDV("true");
+    my $sys_user_group = $sn->table("sys_user_group")->setDV("true");
     my $grp = $sys_user_group->getRecord(name => "Network Support");
     print "manager=", $grp->{manager}, "\n";
 
-    my $sys_user_group = $session->table("sys_user_group")->setDV("all");
+    my $sys_user_group = $sn->table("sys_user_group")->setDV("all");
     my $grp = $sys_user_group->getRecord(name => "Network Support");
     print "manager=", $grp->{dv_manager}, "\n";
     
@@ -1241,6 +1372,9 @@ B<Examples>
 
 sub setDV {
     my ($self, $dv) = @_;
+    $dv = ''     if $dv eq '0';
+    $dv = 'true' if $dv eq '1';
+    $dv = 'all'  if $dv eq '2';
     $self->{dv} = $dv;
     return $self;
 }
@@ -1251,31 +1385,6 @@ sub setTrace {
     return $self;
 }
 
-sub setChunk {
-    my $self = shift;
-    $self->{chunk} = shift || $DEFAULT_CHUNK;
-    return $self;
-}
-
-=head2 setTimeout
-
-Set the value of the SOAP Web Services client HTTP timeout.
-In addition, you may need to increase the ServiceNow system property
-C<glide.soap.request_processing_timeout>.
-    
-B<Syntax>
-
-    $table->setTimeout($seconds);
-    
-=cut
-
-sub setTimeout {
-    my ($self, $seconds) = @_;
-	$seconds = 180 unless defined($seconds) && $seconds =~ /\d+/;
-	$self->{client}->transport()->timeout($seconds);
-	return $self;
-}
-
 sub getSchema {
     my $self = shift;
     return $self->{wsdl} if $self->{wsdl};
@@ -1352,19 +1461,19 @@ In otherwords, all items which are one level downstream.
 First we query C<cmdb_rel_ci> to retrieve a list of 
 records for which this item is the parent.
 C<@relData> is this list of C<cmdb_rel_ci> records.
+If there are more than 250 records, C<fetchAll> will loop internally 
+until all records have been retrieved.
 
     my @relData = $cmdb_rel_ci->query(parent => $parentKey)->fetchAll();
 
 Next we extract the C<child> field from each of these C<cmdb_rel_ci>
-records to create a new list of sys_ids   
+records to create a new list of sys_ids.
 
     my @childKeys = map { $_->{child} } @relData;
 
 C<@childKeys> now contains a lists of sys_ids for configuration items.
 We convert this list of keys into a query object
 and fetch the records from the C<cmdb_ci> table.
-If there are more than 250 records, C<fetchAll> will loop internally 
-until all records have been retrieved.
     
     @childRecs = $cmdb_ci->asQuery(@childKeys)->fetchAll();
     foreach my $childRec (@childRecs) {
@@ -1375,9 +1484,9 @@ until all records have been retrieved.
 
 sub new {
     my ($pkg, $table) = @_;
+    my $session = $table->{session};
     return bless {
         table => $table,
-        chunk => $table->{chunk},
         count => 0,
         index => 0
     }, $pkg;
@@ -1415,7 +1524,7 @@ sub exclude {
 =head2 fetch
 
 Fetch the next chunk of records from a table.
-Returns a list of hashes.
+Returns a list of hash references.
 
 This method calls L</getRecords> to retrieve the records.
 An optional parameter allows specification of the number of records to be retrieved.
@@ -1425,15 +1534,16 @@ If there are no remaining records, then an empty list will be returned.
 
 B<Syntax>
     
-    my @recs = $query->fetch();
-    my @recs = $query->fetch($numrecs);
+    @recs = $query->fetch();
+    @recs = $query->fetch($numrecs);
 
 B<Example>
 
 This example prints the names of all active users.
 Each call to L</getRecords> returns up to 100 users.
 
-    my $query = $sn->table("sys_user")->query(active => "true", __order_by => "name");
+    my $sys_user = $sn->table("sys_user);
+    $sys_user->query(active => "true", __order_by => "name");
     while (@recs = $query->fetch(100)) {
         foreach my $rec (@recs) {
             print $rec->{name}, "\n";
@@ -1444,12 +1554,13 @@ Each call to L</getRecords> returns up to 100 users.
 
 sub fetch {
     my ($self, $chunk) = @_;
-    $chunk = $self->{chunk} unless $chunk;
+    my $table = $self->{table};
+    my $session = $table->{session};    
+    $chunk = $session->{fetch} unless $chunk;
     my @result;
     my $first = $self->{index};
     my $count = $self->{count};
     if ($first < $count) {
-        my $table = $self->{table};
         my $last = $first + $chunk - 1;
         my %params = %{$self->{params}};
         $params{__limit} = $chunk if $chunk;
@@ -1463,23 +1574,25 @@ sub fetch {
 
 =head2 fetchAll
 
-Fetch all the records in a Query by calling L</fetch> repeatedly
+This method fetches all the records in a Query by calling L</fetch> repeatedly
 until there are no more records.
-Returns a list of hashes.
-In a scalar context it returns a reference to an array of hashes.
+It takes an optional argument which is the number of records 
+to be retrieved B<per fetch>.
+It returns a list of hash references.
 
 B<Syntax>
 
-    my @recs = $query->fetchAll();
-    my @recs = $query->fetchAll($chunkSize);
+    @recs = $query->fetchAll();
+    @recs = $query->fetchAll($numrecs);
     
 B<Example>
 
 This example prints the names of all active users.
-Each call to L</getRecords> returns up to 250 users
-since no chunk size was specified and the default is 250.
+Each call to L</getRecords> returns up to 250 records
+since no fetch size was specified and 250 is the default.
 
-    my @recs = $sn->table("sys_user")->query(active => "true", __order_by => "name")->fetchAll();
+    my $sys_user = $sn->table("sys_user);
+    my @recs = $sys_user->query(active => "true", __order_by => "name")->fetchAll();
     foreach my $rec (@recs) {
          print $rec->{name}, "\n";
     }
@@ -1488,7 +1601,6 @@ since no chunk size was specified and the default is 250.
 
 sub fetchAll {
     my ($self, $chunk) = @_;
-    $chunk = $self->{chunk} unless $chunk;
     my @result = ();
     while (my @recs = $self->fetch($chunk)) {
         push @result, @recs;
@@ -1571,38 +1683,6 @@ sub include {
     return $self;
 }
 
-=head2 setChunk
-
-This method sets the default chunk size
-(number of records per fetch)
-for subsequent calls to L</fetch> or L</fetchAll>.
-It returns a reference to the modified Query object.
-
-If C<setChunk> is not specified then 
-the default is 250 records per fetch.
-
-B<Syntax>
-
-    $query->setChunk($numrecs);
-
-B<Example>
-
-    my $rel_tbl = $sn->table("cmdb_rel_ci");
-    my @rel_data = $rel_tbl->query()->setChunk(1000)->fetchAll();
- 
-=cut
-
-sub setChunk {
-    my $self = shift;
-    $self->{chunk} = shift || $DEFAULT_CHUNK;
-    return $self;
-}
-
-sub getChunk {
-    my $self = shift;
-    return $self->{chunk};
-}
-
 sub setIndex {
     my $self = shift;
     $self->{index} = shift;
@@ -1613,33 +1693,31 @@ sub setIndex {
 
 =head1 DIAGNOSTICS
 
-The fourth (optional) argument to the L</ServiceNow> function 
-is an integer trace level which can be helpful for debugging.
 Sometimes, when you are developing a new script,
 it seems to hang at a certain point, 
 and you just want to know what it is doing.
-Set the trace level to 1 to print a single line for each Web Services call.
-Set the trace level to 2 to print the complete XML result for each call.
-Set the trace level to 0 (default) to disable this feature.
+You can enable tracing of Web Service calls
+by setting the C<trace> parameter
+in the L</ServiceNow> function as follows.
 
-You can also enable or disable tracing for a table by 
-calling C<setTrace> on the object.
+    my $sn = ServiceNow($instance, $username, $password, trace => 1);
+
+Set trace to 1 to print a single line for each Web Services call.
+Set trace to 2 to print the complete XML result for each call.
 
-    $table->setTrace(2);
-    
 If you want even more, then add the following to your code.
-This will cause SOAP:Lite to dump the HTTP headers
+This will cause L<SOAP::Lite> to dump the HTTP headers
 and contents for all messages, both sent and received.
 
     SOAP::Lite->import(+trace => 'debug');
 
 =head1 AUTHOR
 
-Giles Lewis 
+Giles Lewis <gflewis@cpan.org>
 
 =head1 ACKNOWLEDGEMENTS
 
-Greg George, author of ServiceNow::Simple,
+Greg George, author of L<ServiceNow::Simple>,
 from which a number of ideas were sourced. 
 
 =head1 LICENSE
